"""added fields from mdm

Peek Plugin Database Migration Script

Revision ID: b3f41d4bc7c2
Revises: de24e18d0e6e
Create Date: 2023-10-06 14:37:17.736457

"""

# revision identifiers, used by Alembic.
revision = "b3f41d4bc7c2"
down_revision = "de24e18d0e6e"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


__affectedTableName = "DeviceInfo"
__affectedSchemaName = "core_device"
__affectedColumnNames = [
    "mdmDeviceName",
    "mdmDeviceSerialNumber",
    "mdmDeviceAssetId",
    "mdmDeviceAllocatedTo",
]


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    for columnName in __affectedColumnNames:
        column = sa.Column(
            columnName,
            sa.String(),
            server_default="",
            nullable=True,
        )
        op.add_column(
            __affectedTableName, column=column, schema=__affectedSchemaName
        )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    for columnName in __affectedColumnNames:
        op.drop_column(
            __affectedTableName, columnName, schema=__affectedSchemaName
        )
    # ### end Alembic commands ###



from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class ClientSettingsTuple(Tuple):
    """Client Settings Tuple

    This tuple is for the client UI settings.

    """

    __tupleType__ = deviceTuplePrefix + "ClientSettingsTuple"

    fieldEnrollmentEnabled: bool = TupleField()
    officeEnrollmentEnabled: bool = TupleField()
    slowNetworkBandwidthMetricThreshold: int = TupleField()
    offlineMasterSwitchEnabled: bool = TupleField()

    offlineCacheSyncSeconds: int = TupleField()
    checkBandwidthSeconds: int = TupleField()
    abortRetrySeconds: int = TupleField()
    pauseTimeoutSeconds: int = TupleField()
    sendStateToServerSeconds: int = TupleField()



"""add lastBandwidthMetric

Peek Plugin Database Migration Script

Revision ID: 97d44e5aeb18
Revises: 4b0d6309e556
Create Date: 2023-02-23 15:39:49.570045

"""

# revision identifiers, used by Alembic.
revision = "97d44e5aeb18"
down_revision = "4b0d6309e556"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DeviceInfo",
        sa.Column("lastBandwidthMetric", sa.Integer(), nullable=True),
        schema="core_device",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DeviceInfo", "lastBandwidthMetric", schema="core_device")
    # ### end Alembic commands ###



import json
import logging
from collections import Counter
from datetime import datetime
from datetime import timedelta
from pathlib import Path

import pytz
from twisted.internet.task import LoopingCall
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.DeferUtil import vortexLogFailure
from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix
from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_plugin_base.LoopingCallUtil import peekCatchErrbackWithLogger

logger = logging.getLogger(__name__)


@addTupleType
class DeviceMetricsTuple(Tuple):
    __tupleType__ = deviceTuplePrefix + "DeviceMetricsTuple"

    lastUpdateDate: datetime = TupleField()
    totalDevices: int = TupleField()
    devicesOnlineLast365Days: int = TupleField()
    devicesOnlineLast30Days: int = TupleField()
    devicesOnlineLast7Days: int = TupleField()
    devicesOnlineLast24Hours: int = TupleField()
    deviceTypeCount: dict[str, int] = TupleField()


class MetricsController:
    LOOPING_CALL_PERIOD = 30.0 * 60  # 30 minutes
    """Collects and writes system metrics to a JSON file."""

    def __init__(self, writeDir: Path, ormSessionCreator):
        self._writeDir = writeDir
        self._ormSessionCreator = ormSessionCreator
        self._loopingCall = LoopingCall(
            peekCatchErrbackWithLogger(logger)(self.writeMetrics)
        )

    def start(self):
        d = self._loopingCall.start(self.LOOPING_CALL_PERIOD, now=True)
        d.addErrback(vortexLogFailure, logger, consumeError=True)

    def shutdown(self):
        if self._loopingCall is not None:
            self._loopingCall.stop()
            self._loopingCall = None

    @deferToThreadWrapWithLogger(logger)
    def writeMetrics(self) -> None:
        """Collect and write system metrics to a JSON file.

        Raises:
            OSError: If there are file permission issues
            psutil.Error: If there are errors collecting system metrics
        """
        try:
            metrics = self._collectMetrics()
            jsonFilePath = self._writeDir / "metrics.json"

            # Convert datetime to string for JSON serialization
            metricsDict = metrics.tupleToRestfulJsonDict()

            with jsonFilePath.open("w") as f:
                json.dump(metricsDict, f, indent=4)

        except Exception as e:
            logger.exception(f"Error writing metrics: {e}")
            raise

    def _addDeviceMetrics(self, metrics: DeviceMetricsTuple) -> None:
        """Get memory-related metrics including swap usage."""
        ormSession = self._ormSessionCreator()
        try:

            devices = list(
                ormSession.query(
                    DeviceInfoTable.lastOnline, DeviceInfoTable.deviceType
                )
            )
            now = datetime.now(pytz.utc)

            # Time thresholds
            one_year_ago = now - timedelta(days=365)
            one_month_ago = now - timedelta(days=30)
            one_week_ago = now - timedelta(days=7)
            today_start = now - timedelta(days=1)

            # Calculate metrics
            metrics.totalDevices = len(devices)
            metrics.devicesOnlineLast365Days = sum(
                1 for device in devices if device.lastOnline >= one_year_ago
            )
            metrics.devicesOnlineLast30Days = sum(
                1 for device in devices if device.lastOnline >= one_month_ago
            )
            metrics.devicesOnlineLast7Days = sum(
                1 for device in devices if device.lastOnline >= one_week_ago
            )
            metrics.devicesOnlineLast24Hours = sum(
                1 for device in devices if device.lastOnline >= today_start
            )
            metrics.deviceTypeCount = dict(
                Counter(device.deviceType for device in devices)
            )

        finally:
            ormSession.close()

    def _collectMetrics(self) -> DeviceMetricsTuple:
        """Collect all system metrics and return as an OsMetricsTuple."""

        metrics = DeviceMetricsTuple(lastUpdateDate=datetime.now(pytz.utc))
        self._addDeviceMetrics(metrics)
        return metrics



from datetime import datetime

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class OfflineCacheLoaderStatusTuple(Tuple):
    """Offline Cache Status Tuple

    This tuple represents the load status of the mobile device.

    """

    __tupleType__ = deviceTuplePrefix + "OfflineCacheLoaderStatusTuple"

    pluginName: str = TupleField()
    indexName: str = TupleField()
    loadingQueueCount: int = TupleField()
    totalLoadedCount: int = TupleField()
    lastCheckDate: datetime = TupleField()
    initialFullLoadComplete: bool = TupleField()



import logging
from random import random

from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.PayloadEndpoint import PayloadEndpoint
from vortex.PayloadEnvelope import PayloadEnvelope
from vortex.VortexABC import SendVortexMsgResponseCallable

from peek_core_device._private.PluginNames import deviceFilt
from peek_core_device._private.tuples.BandwidthTestTuple import (
    BandwidthTestTuple,
)

logger = logging.getLogger(__name__)

deviceBandwidthTestFilt = dict(key="deviceBandwidthTestFilt")
deviceBandwidthTestFilt.update(deviceFilt)


class BandwidthTestController:
    def __init__(self):
        self._vortexMsg = None
        self._cachedEncodedPayload = None

        self._endpoint = None

    @inlineCallbacks
    def start(self):
        self._endpoint = PayloadEndpoint(deviceBandwidthTestFilt, self._process)
        yield self._createTestData()

    def shutdown(self):
        if self._endpoint:
            self._endpoint.shutdown()
            self._endpoint = None

    @deferToThreadWrapWithLogger(logger)
    def _createTestData(self):
        BASE64_SIZE = 1.3
        PACKET_SIZE = int((100.0 * 1024.0) / BASE64_SIZE)

        testData = ""
        while len(testData) < PACKET_SIZE:
            testData += str(random())

        logger.debug(f"Generated testData of size {len(testData)}")

        tuple_ = BandwidthTestTuple(testData=testData)

        self._vortexMsg = PayloadEnvelope(
            filt=deviceBandwidthTestFilt,
            encodedPayload=Payload(
                filt=deviceBandwidthTestFilt, tuples=[tuple_]
            ).toEncodedPayload(compressionLevel=0),
        ).toVortexMsg(base64Encode=False)

    def _process(
        self, sendResponse: SendVortexMsgResponseCallable, *args, **kwargs
    ):
        sendResponse(self._vortexMsg)



import logging
from typing import Union

from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.PayloadEnvelope import PayloadEnvelope
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_core_device._private.server.controller.OfflineCacheController import (
    OfflineCacheController,
)
from peek_core_device._private.tuples.DeviceCacheStatusTuple import (
    DeviceCacheStatusTuple,
)

logger = logging.getLogger(__name__)


class OfflineCacheCombinedStatusTupleProvider(TuplesProviderABC):
    def __init__(self, offlineCacheController: OfflineCacheController):
        self._offlineCacheController = offlineCacheController

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        deviceToken = tupleSelector.selector.get("deviceToken")

        encodedPayload = (
            self._offlineCacheController.lastCacheStatusEncodedPayload(
                deviceToken
            )
        )

        # Create the vortex message
        return PayloadEnvelope(
            filt=filt, encodedPayload=encodedPayload
        ).toVortexMsg()



import logging
from typing import Union

import sqlalchemy
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_core_device._private.tuples.OfflineCacheSettingTuple import (
    OfflineCacheSettingTuple,
)

logger = logging.getLogger(__name__)


class OfflineCacheSettingTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        deviceToken = tupleSelector.selector.get("deviceToken")

        ormSession = self._ormSessionCreator()
        try:
            try:
                deviceTuple = (
                    ormSession.query(DeviceInfoTable)
                    .filter(DeviceInfoTable.deviceToken == deviceToken)
                    .one()
                )

            except sqlalchemy.orm.exc.NoResultFound:
                deviceTuple = DeviceInfoTable(isOfflineCacheEnabled=False)

            tuples = [
                OfflineCacheSettingTuple(
                    offlineEnabled=deviceTuple.isOfflineCacheEnabled
                )
            ]

            # Create the vortex message
            return (
                Payload(filt, tuples=tuples).makePayloadEnvelope().toVortexMsg()
            )

        finally:
            ormSession.close()



from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix
from peek_core_device._private.tuples.OfflineCacheLoaderStatusTuple import (
    OfflineCacheLoaderStatusTuple,
)
from peek_core_device._private.tuples.OfflineCacheStatusTuple import (
    OfflineCacheStatusTuple,
)


@addTupleType
class OfflineCacheCombinedStatusTuple(Tuple):
    """Offline Cache Status Action

    This tuple is sent from the client to the server periodically when
    an update cycle finishes

    """

    __tupleType__ = deviceTuplePrefix + "OfflineCacheCombinedStatusTuple"

    deviceToken: str = TupleField()
    loaderStatusList: list[OfflineCacheLoaderStatusTuple] = TupleField([])
    offlineCacheStatus: OfflineCacheStatusTuple = TupleField()
